/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.processEditor
{
  import flash.display.DisplayObject;
  import flash.events.MouseEvent;
  import flash.geom.Point;

  import mx.controls.Image;
  import mx.events.FlexEvent;
  import mx.states.AddChild;
  import mx.states.SetEventHandler;
  import mx.states.State;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
  import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
  import org.ow2.orchestra.designer.bpmn.views.utils.GraphicUtils;
  import org.ow2.orchestra.designer.events.ProcessModificationEvent;

  public class ProcessEditorCreatingElementStateHelper extends ProcessEditorFlowStateHelper {


    public function ProcessEditorCreatingElementStateHelper(processEditorView:ProcessEditorView) {
      super(processEditorView);
    }

    /**
     * Generate "creating gateway" state.
     * In the creating flow state:
     * - the flow is drawn, from source element to mouse cursor
     * - on mouse move (while the button is down), the flow is updated : the head of the flow follows the mouse cursor.
     * - on clic, if the clic is on a compatible element, the flow is drawn between source and target elements . This view goes back to the "default" state.
     */
    public function addCreatingElementState():void {

      var state:State = new State();
      state.name = "creatingElement";
      processEditorView.states.push(state);

      var addArrowChild:AddChild = new AddChild(processEditorView, processEditorView.arrow);
      var addGatewayChild:AddChild = new AddChild(processEditorView, processEditorView.imageOfElementToAdd);
      state.overrides.push(addArrowChild);
      state.overrides.push(addGatewayChild);

      // handler for mouse move event
      var mouseMoveListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_MOVE);
      mouseMoveListener.handlerFunction = creatingElementMouseMove;
      state.overrides.push(mouseMoveListener);

      // handler for mouse click event
      var mouseClickListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.CLICK);
      mouseClickListener.handlerFunction = creatingElementMouseClick;
      state.overrides.push(mouseClickListener);

      // handler for state enter: resets the arrow position
      state.addEventListener(FlexEvent.ENTER_STATE, creatingElementStateEnter);
      state.addEventListener(FlexEvent.EXIT_STATE, creatingGatewayStateExit);
    }

    /**
     * handler for mouse move events when this view is in the "creating flow" state.
     */
    private function creatingElementStateEnter(event:FlexEvent):void {
      processEditorView.hideElementContextPanel();

      // Get mouse position.
      var mousePoint:Point = new Point(processEditorView.mouseX, processEditorView.mouseY);

      creatingElementMoveArrow(mousePoint);
    }

    private function creatingGatewayStateExit(event:FlexEvent):void {
    }

    /**
     * handler for mouse move events when this view is in the "creating flow" state.
     */
    private function creatingElementMouseMove(event:MouseEvent):void {
      // Get mouse event position. The coordinates are relative to the target element
      var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
      // Transform the coordinates to be relative to this view
      var mousePoint:Point = processEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

      creatingElementMoveArrow(mousePoint);
    }

    private function creatingElementMoveArrow(mousePoint:Point):void {
      // Get coordinates of the center of the source element
      var element:AbstractElementWithPosition = processEditorView.processEditorModel.selectedElements.getItemAt(0) as
        AbstractElementWithPosition;
      var sourceView:AbstractElementView = processEditorView.processView.getElementViewByModel(element);
      var sourceImage:DisplayObject = sourceView.borderImage;
      var sourcePoint:Point = GraphicUtils.findIntersection(
        sourceImage,
        processEditorView.processView.globalToLocal(sourceView.localToGlobal(new Point(sourceImage.x + sourceImage.width
          / 2, sourceImage.y + sourceImage.height / 2))),
        mousePoint,
        processEditorView.processView);

      processEditorView.imageOfElementToAdd.x = mousePoint.x - processEditorView.imageOfElementToAdd.width / 2;
      processEditorView.imageOfElementToAdd.y = mousePoint.y - processEditorView.imageOfElementToAdd.height / 2;
      var targetPoint:Point = GraphicUtils.findIntersection(
        processEditorView.imageOfElementToAdd,
        mousePoint,
        processEditorView.processView.globalToLocal(sourceView.localToGlobal(new Point(sourceImage.x, sourceImage.y))),
        processEditorView.processView);

      processEditorView.arrow.setSourceAndTarget(sourcePoint, targetPoint);
    }

    /**
     * handler for mouse click events when this view is in the "creating element" state.
     * Select the element under the mouse cursor and create a flow to this element.
     * The view goes back to the "default" state.
     */
    private function creatingElementMouseClick(event:MouseEvent):void {
      // Get mouse event position. The coordinates are relative to the target element
      var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
      // Transform the coordinates to be relative to this view
      var mousePoint:Point = processEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

      // check if mouse is on a element view or on the canvas.
      trace("creatingElementWithSequenceFlowMouseClick");

      // create a new element and a new sequenceFlow
      var elementSource:AbstractElementWithPosition =
          processEditorView.processEditorModel.selectedElements.getItemAt(0) as AbstractElementWithPosition;


      new ProcessModificationEvent(ProcessModificationEvent.PROCESS_ELEMENT_ADD_WITH_SEQUENCE,
        this.processEditorView.processEditorModel,
        processEditorView.modelOfElementToAdd,
        mousePoint.x - processEditorView.imageOfElementToAdd.width / 2,
        mousePoint.y - processEditorView.imageOfElementToAdd.height / 2,
        elementSource).dispatch();
      // go to default state
      processEditorView.currentState = "default";
    }
  }
}