/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


package org.ow2.orchestra.designer.view.processEditor {
  import flash.display.DisplayObject;
  import flash.events.MouseEvent;
  import flash.geom.Point;
  import flash.geom.Rectangle;

  import mx.collections.ArrayCollection;
  import mx.states.AddChild;
  import mx.states.SetEventHandler;
  import mx.states.State;

  import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
  import org.ow2.orchestra.designer.bpmn.views.ProcessPoolView;
  import org.ow2.orchestra.designer.events.ProcessSelectionElementsEvent;

  public class ProcessEditorSelectingStateHelper extends ProcessEditorFlowStateHelper {

    public function ProcessEditorSelectingStateHelper(processEditorView:ProcessEditorView) {
      super(processEditorView);
    }

    /**
     * Generate "selecting" state.
     * In the selecting state:
     * - the selection rectangle is shown, with zero width and height, and the top left corner at the current mouse position
     * - on mouse move (while the button is down), the rectangle bottom right corner follows the mouse cursor.
     * - on mouse up, all the process model elements attached to view in the selection rectangle becomes selected. This view goes back to the "default" state.
     * - when the mouse leaves this view, the view goes back to the "default" state. No changes are done to the list of selected elements.
     */
    public function addSelectingState():void {
      var state:State = new State();
      state.name = "selecting";
      processEditorView.states.push(state);

      // show selection rectangle
      var addSelectionRectChild:AddChild = new AddChild(processEditorView, processEditorView.selectionRect);
      state.overrides.push(addSelectionRectChild);

      // handler for mouse up event
      var mouseUpListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_UP);
      mouseUpListener.handlerFunction = selectingMouseUp;
      state.overrides.push(mouseUpListener);

      // handler for mouse move event
      var mouseMoveListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_MOVE);
      mouseMoveListener.handlerFunction = selectingMouseMove;
      state.overrides.push(mouseMoveListener);

    }

    /**
     * handler for mouse move event when this view is in the "selected" state.
     * Makes the rectangle bottom right corner follows the mouse cursor.
     */
    private function selectingMouseMove(event:MouseEvent):void {
      processEditorView.hideElementContextPanel();

      // Get mouse event position. The coordinates are relative to the target element
      var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
      // Transform the coordinates to be relative to this view
      var mousePoint:Point = processEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

      // update rectangle width and height to follow the mouse cursor
      processEditorView.selectionRect.width = mousePoint.x - processEditorView.selectionRect.x;
      processEditorView.selectionRect.height = mousePoint.y - processEditorView.selectionRect.y;
    }


    /**
     * handler for mouse up event when this view is in the "selected" state.
     * All the process model elements attached to view in the selection rectangle becomes selected.
     * This view goes back to the "default" state.
     */
    private function selectingMouseUp(event:MouseEvent):void {
      // get selection rectangle coordinates
      var selectionX:int = processEditorView.selectionRect.x;
      var selectionY:int = processEditorView.selectionRect.y;
      var selectionWidth:int = processEditorView.selectionRect.width;
      var selectionHeight:int = processEditorView.selectionRect.height;
      // if width is negative, change coordinates to have the same rectangle with positive width
      if (selectionWidth < 0) {
        selectionX = selectionX + selectionWidth;
        selectionWidth = -selectionWidth;
      }
      // if height is negative, change coordinates to have the same rectangle with positive height
      if (selectionHeight < 0) {
        selectionY = selectionY + selectionHeight;
        selectionHeight = -selectionHeight;
      }
      // Create rectangle for the selection area
      var selection:Rectangle = new Rectangle(selectionX, selectionY, selectionWidth, selectionHeight);

      // find list of elements that are contained in the selection area
      var listSelectedElements:ArrayCollection = new ArrayCollection();
      //Try first to select by click
      for (var index:int = processEditorView.processView.getChildren().length - 1; index >= 0; index--) {
        var child:DisplayObject = processEditorView.processView.getChildren()[index];
        if (!(child is ProcessPoolView) && child is AbstractElementView && AbstractElementView(child).containsRectangle(selection, processEditorView)) {
          // the process element view is contained in the selection area,
          // the attached model should be selected.
          listSelectedElements.addItem((child as AbstractElementView).model);
          //For the current implementation stop at the first match
          //TODO: choose the most valuable object to select.
          break;
        }
      }
      if (listSelectedElements.length == 0) {
        //Else try to select by selection area
        for each (var object:DisplayObject in processEditorView.processView.getChildren()) {
          if (!(object is ProcessPoolView) && object is AbstractElementView) {
            if (AbstractElementView(object).includeInto(selection, processEditorView)) {
              // the process element view is contained in the selection area,
              // the attached model should be selected.
              listSelectedElements.addItem((object as AbstractElementView).model);
            }
          }
        }
      }
      // generate event for element selection.
      new ProcessSelectionElementsEvent(
              ProcessSelectionElementsEvent.PROCESS_ELEMENTS_SELECT,
              processEditorView.processEditorModel,
              listSelectedElements).dispatch();
      // go back to default state
      processEditorView.currentState = "default";
    }


  }
}
